/**
 * 将数值四舍五入(保留2位小数)后格式化成金额形式
 *
 * @param num 数值(Number或者String)
 * @return String --金额格式的字符串,如'1,234,567.45'
 * @type
 */
function formatCurrency(num,decimal) {
  num = num.toString().replace(/\$|\,/g,'');
  if(isNaN(num))
    num = "0";
  var sign = (num == (num = Math.abs(num)));
  var cents = 0;
  if(decimal > 0)
  {
    num = Math.floor(num*100+0.50000000001);
    cents = num%100;
    num = Math.floor(num/100).toString();
    if(cents<10)
      cents = "0" + cents;
  }
  else
  {
    num = num.toString();
  }
  for (var i = 0; i < Math.floor((num.length-(1+i))/3); i++)
    num = num.substring(0,num.length-(4*i+3))+','+ num.substring(num.length-(4*i+3));

  var result = ((sign)?'':'-') + num;
  if(decimal > 0)
  {
    return (result + '.' + cents);
  }

  return result;
}
function numFormat(num, fmt) {
  try {

    if(num == -999) return "--";

    var number = parseFloat(num);
    if(isNaN(number)) return num;

    if(!fmt) return num;

    // 百分比格式
    if(fmt == ".2f%") {
      return number.toFixed(2) + "%";
    }
    else if(fmt == ".1f%") {
      return number.toFixed(1) + "%";
    }

    var n = fmt.match(/\d/);
    if(n) {
      n = n[0];
      return number.toFixed(n);
    } else {
      return num;
    }
  } catch(e) {
    return num;
  }
}
function ownnumFormat(num, fmt) {
  try {

    if(num == -999) return "--";

    var number = parseFloat(num);
    if(isNaN(number)) return num;

    if(!fmt) return num;

    // 百分比格式
    if(fmt == ".2f%") {
      return number.toFixed(2);
    }
    else if(fmt == ".1f%") {
      return number.toFixed(1);
    }

    var n = fmt.match(/\d/);
    if(n) {
      n = n[0];
      return number.toFixed(n);
    } else {
      return num;
    }
  } catch(e) {
    return num;
  }
}
function unitConvert(num) {
    var moneyUnits = ["元", "万元", "亿元", "万亿"];
    var dividend = 10000;
    var curentNum = num?num:0;
    var curentUnit = moneyUnits[0];
    for (var i = 0; i < 4; i++) {
        curentUnit = moneyUnits[i];
        if (strNumSize(curentNum) < 5) {
            break;
        }
        curentNum = curentNum / dividend;
    }

    return parseFloat(curentNum).toFixed(2)+curentUnit;
}
function strNumSize(tempNum) {
    var stringNum = tempNum.toString();
    var index = stringNum.indexOf(".");
    var newNum = stringNum;
    if (index != -1) {
        newNum = stringNum.substring(0, index);
    }
    return newNum.length;
}
function formatDate(date) {
  // 20160517 => 2016-05-17
  if(typeof (date) == 'undefined'  || date.trim() == "" || date.length < 6)
    return date;
  date = date.replace(/-/g,'');
  return date.substr(0, 4) + '-' + date.substr(4, 2) +  '-' + date.substr(6, 2);

}


function loadScript(url, callback){

  var xhr = new XMLHttpRequest();
  xhr.open("get", url, true);
  xhr.onreadystatechange = function(){
    if (xhr.readyState == 4){
      // xhr.responseText 判断是否有内容返回
      if (xhr.status >= 200 && xhr.status < 300 || xhr.status == 304 || xhr.responseText != ""){
        var script = document.createElement ("script");
        script.type = "text/javascript";
        var script_text = "//@ sourceURL=" + url + "\n" + xhr.responseText;
        script.text = script_text;
        // script.text = xhr.responseText;
        document.body.appendChild(script);
        callback();
      }
    }
  };
  xhr.send(null);
}

// 自定义弹出框和确认框
window.alert = function (str, callback, title) {
  _showDialog(1, str, callback, null, title);
};
window.confirm = function (str, callback_ok , callback_cancel, title, oktext,canceltext) {
    _showDialog(0, str, callback_ok, callback_cancel,title, oktext, canceltext);
};
function _showDialog(is_alert, str, callback_ok, callback_cancel, title,oktext,canceltext) {

  // 这里做APP中的弹出框兼容
  // 微信中的弹出框是在 wrapper.html 定义的，这里需要在APP中动态添加

  // 非 wrapper 页面，并且页面不在 iframe 中
  if(window.top == window && window.location.href.indexOf("wrapper.html") < 0) {
    // 没有添加提示框在页面dom中
    if($(".alert-dialog").length <= 0) {

      var cssStyleStr = " \
        .alert-dialog { \
          z-index: 9999; \
          width:100%; \
          height:100%; \
          background-color: rgba(10,10,10,0.5); \
          color:#fff; \
          display: none; \
          bottom:0px;position:fixed; \
        } \
        .alert-container { \
          width:100%; \
          height:100%; \
          display:flex; \
          display: -webkit-flex; \
        } \
        .alert-panel { \
          width:82%; \
          background-color: #fff; \
          border-radius:10px; \
          display: -webkit-flex; \
          display: flex; \
          margin:auto auto; \
          color:#333; \
          flex-direction: column; \
          align-items:center; \
          -webkit-flex-direction: column; \
          -webkit-align-items:center; \
        } \
        .alert-title, .alert-content { \
          display: -webkit-flex; \
          display: flex; \
        } \
        .alert-title { \
          height:48px; \
          line-height:48px; \
          font-size:18px; \
          margin: 6px auto; \
          border-bottom: 1px solid #E8E8E8; \
          width: 80%; \
          display: -webkit-flex; \
          display: flex; \
          justify-content: center; \
          -webkit-justify-content: center; \
        } \
        .alert-content { \
          margin: 0 0; \
          width: 85%;\
          align-items: center;\
          -webkit-align-items: center;\
          padding-top:10px;\
          color: #666; \
          line-height: 28px; \
          font-size: 15px; \
          min-height: 100px; \
          max-height: 300px; \
          overflow: auto; \
          display: flex; \
          display: -webkit-flex; \
          flex-direction:column; \
          -webkit-flex-direction:column; \
          // justify-content: center; \
          // -webkit-justify-content: center; \
          word-break:break-all;\
          word-wrap:break-word;\
        } \
        .alert-buttons { \
          height:80px; \
          width:76%; \
          display: -webkit-flex; \
          display: flex; \
        } \
       .alert-button { \
          display: -webkit-flex; \
          -webkit-flex: 1 1 auto; \
          -webkit-align-self: center; \
          -webkit-justify-content: center; \
          display: flex; \
          flex: 1 1 auto; \
          align-self: center; \
          -webkit-align-self:center;\
          justify-content: center; \
        } \
        .alert-btn { \
          height: 36px; \
          line-height: 36px; \
          font-size:16px; \
          width: 80%;\
          color:white;\
          text-align: center;\
          border-radius: 4px;\
          cursor: pointer; \
        } \
        .alert-btn-no { \
          background-color:#C5C5C5;\
        } \
        .alert-btn-yes { \
          background-color:#F03838;\
        } \
      ";

      $("head").append("<style type='text/css'>" + cssStyleStr + "</style>");

      var domStr = "<div class='alert-dialog'> "+
      "    <div class='alert-container'> "+
      "      <div class='alert-panel'> "+
      "        <div class='alert-title'>温馨提示</div> "+
      "        <div class='alert-content'> "+
      "        </div> "+
      "        <div class='alert-buttons'> "+
      "          <div class='alert-button alert-button-no'><div class='alert-btn alert-btn-no'>"+(canceltext?canceltext:"取消")+"</div></div> "+
      "          <div class='alert-button alert-button-yes'><div class='alert-btn alert-btn-yes'>"+(oktext?oktext:"确认")+"</div></div> "+
      "        </div> "+
      "      </div> "+
      "    </div> "+
      "  </div> ";

      $("body").append(domStr);
    }

    appShowDialog(is_alert, str, callback_ok, callback_cancel, title,oktext,canceltext);

  } else {

    if (window.history && window.history.pushState) {
      $(window).on('popstate', function () {
        // var json=window.history.state;
        window.parent.closeDialog(callback_cancel);
      });
      //history.pushState(state, title, url)
      window.history.pushState('forward', "title", window.location);
    }
    window.parent.showDialog(is_alert, str, callback_ok, callback_cancel, title);
  }

}
function appShowDialog(is_alert, str, callback_ok, callback_cancel, title,oktext,canceltext) {
  if(title == '' || title == null) {
    title = '温馨提示';
  }
  $(".alert-title").html(title);
  $(".alert-content").html(str);
  $(".alert-btn").unbind("click");
  $(".alert-btn-no").html(canceltext ? canceltext:"取消");
  $(".alert-btn-yes").html(oktext ? oktext:"确定");
  if(is_alert) {
    $(".alert-button-no").hide();
  } else {
    $(".alert-button-no").show();
  }

  $(".alert-btn-no").click(function () {
    $(".alert-dialog").fadeOut("fast",callback_cancel);
    // window.history.back();
  });
  $(".alert-btn-yes").click(function () {
    $(".alert-dialog").fadeOut("fast", callback_ok);
    // window.history.back();
  });

  $(".alert-dialog").fadeIn("fast");
}


function compareFuncDesc(a,b)
{
  if(a["F143"] == b["F143"])
    return 0;
  if(a["F143"] > b['F143'])
    return -1;
  return 1;
}

// 获取当前登录的账号信息
function getCurrentUser(callback)
{
  __tdxgetAccList(function(list) {

    var curUser;
    try {
      if (typeof list == "string") {
        list = JSON.parse(list);
      }
      if (list.length > 0) {
        if (typeof list[0] == "string") {
          curUser = JSON.parse(list[0]);
        } else {
          curUser = list[0];
        }
      }

    } catch(e) {
      alert('获取当前用户失败');
    }

    callback(curUser);
  });
}

var MyFormatResult = function (data) {

  try {

    if (typeof data == 'string') {
      data = data.replace(/\r\n/ig, "<br>");
      data = data.replace(/\s/g, "");
      data = JSON.parse(data);
    }

    var rt = {};
    rt.ErrorCode = data[0][0];
    rt.ErrorInfo = data[0][1];
    rt.Num = data[0][2];
    rt.Cookies = data[0][3];
    rt.POS = data[0][4];

    if (rt.ErrorCode != 0) {
      return rt;
    }

    // 数据部分
    rt.rows = [];

    var titles = data[1];
    data.shift();
    data.shift();
    data.shift();

    rt.rows = data.reduce(function (prev, curr) {
      // 将扁平数组转换为对象
      prev.push(curr.reduce(function (p, c, index) {
        p[titles[index]] = c;
        return p;
      }, {}));
      return prev;
    }, []);

    return rt;

  } catch (e) {
    return {ErrorCode:-1,ErrorInfo:data};
  }
};

// 某些页面预加载图标
function showPreLoading(){
  var winWidth = 0, winHeight = 0;
  if (window.innerWidth)
    winWidth = window.innerWidth;
  else if ((document.body) && (document.body.clientWidth))
    winWidth = document.body.clientWidth;
// 获取窗口高度
  if (window.innerHeight)
    winHeight = window.innerHeight;
  else if ((document.body) && (document.body.clientHeight))
    winHeight = document.body.clientHeight;
  $(".pre-load").css({"width":winWidth + "px", height:winHeight + "px"});
  $(".pre-load").show();

}
function hidePreLoading() {
  $(".pre-load").hide();
}

function shortDate(d) {
    if(d.length==10){
        d=d.substring(5,10)
    }
    return d;
}
function tap(el, callback, start, end){
  var startX, startY;
  var endX, endY;
  var startTime, endTime;
  el.addEventListener('touchstart',function(e){
    startX = e.changedTouches[0].clientX;
    startY = e.changedTouches[0].clientY;
    startTime = new Date().getTime();
    start && start.call(el, e);
  });
  el.addEventListener('touchend',function(e){
    endX = e.changedTouches[0].clientX;
    endY = e.changedTouches[0].clientY;
    endTime = new Date().getTime();
    if(Math.abs(endX-startX) < 5 && Math.abs(endY-startY) < 5 && endTime-startTime < 300){
      callback && callback.call(el, e);
    }
    end && end.call(el, e)
  })
}
